@@ -2,9 +2,13 @@ import logging
 import os
 import signal
 import threading
+from socket import socket
+from types import FrameType
+from typing import Callable, Dict, List, Optional
 
 import click
 
+from uvicorn.config import Config
 from uvicorn.subprocess import get_subprocess
 
 HANDLED_SIGNALS = (
@@ -16,21 +20,26 @@ logger = logging.getLogger("uvicorn.error")
 
 
 class BaseReload:
-    def __init__(self, config, target, sockets):
+    def __init__(
+        self,
+        config: Config,
+        target: Callable[[Optional[List[socket]]], None],
+        sockets: List[socket],
+    ) -> None:
         self.config = config
         self.target = target
         self.sockets = sockets
         self.should_exit = threading.Event()
         self.pid = os.getpid()
-        self.reloader_name = None
+        self.reloader_name: Optional[str] = None
 
-    def signal_handler(self, sig, frame):
+    def signal_handler(self, sig: signal.Signals, frame: FrameType) -> None:
         """
         A signal handler that is registered with the parent process.
         """
         self.should_exit.set()
 
-    def run(self):
+    def run(self) -> None:
         self.startup()
         while not self.should_exit.wait(self.config.reload_delay):
             if self.should_restart():
@@ -38,7 +47,7 @@ class BaseReload:
 
         self.shutdown()
 
-    def startup(self):
+    def startup(self) -> None:
         message = f"Started reloader process [{self.pid}] using {self.reloader_name}"
         color_message = "Started reloader process [{}] using {}".format(
             click.style(str(self.pid), fg="cyan", bold=True),
@@ -54,8 +63,8 @@ class BaseReload:
         )
         self.process.start()
 
-    def restart(self):
-        self.mtimes = {}
+    def restart(self) -> None:
+        self.mtimes: Dict[str, float] = {}
 
         self.process.terminate()
         self.process.join()
@@ -65,7 +74,7 @@ class BaseReload:
         )
         self.process.start()
 
-    def shutdown(self):
+    def shutdown(self) -> None:
         self.process.join()
         message = "Stopping reloader process [{}]".format(str(self.pid))
         color_message = "Stopping reloader process [{}]".format(
@@ -73,5 +82,5 @@ class BaseReload:
         )
         logger.info(message, extra={"color_message": color_message})
 
-    def should_restart(self):
+    def should_restart(self) -> bool:
         raise NotImplementedError("Reload strategies should override should_restart()")
